# ---------------------------------------------------------------------------
# Copyright 2002 Benjamin Trott
# Copyright 2003 Dwight Shih
#
# Adminstrator Interface for comment Package
# Derived from the Standalone Trackback package released by Benjamin Trott.
#
# This code is released under the Artistic License.
#
# $Id: admin.pm,v 1.7 2005/03/18 23:55:39 dwight Exp $
# ---------------------------------------------------------------------------

package comment::admin;

use strict;
use vars qw($VERSION @ISA);

require POSIX;

use Exporter;
use CGI qw( :cgi :standard );

use trackback::core;

@ISA = qw(Exporter trackback::core);
$VERSION = '1.00';

# ---------------------------------------------------------------------------

sub new {
    my $class = shift;
    my %param = @_;
    my $self  = trackback::core->new(@_);
    bless($self,$class);

    #
    if ( $param{HEADER} ) {
        $self->{HEADER} = $param{HEADER};
    } else {
        $self->{HEADER} = <<HEADER_END;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">

<html>
<head>
<title>Comment Administration</title>
<meta name="robots" content="noindex,nofollow">
<style type="text/css">
body {
    color:black;
    background-color:#CCCCCC;
}
.even {
    color:black;
    background-color:white;
    padding-top: 5px;
    padding-bottom: 5px;
}
.odd {
    color:black;
    background-color:#CCCCCC;
    padding-top: 5px;
    padding-bottom: 5px;
}
</style>
</head>
<body>
HEADER_END
    }

    #
    if ( $param{BODY_FUNCTION} ) {
        $self->{BODY_FUNCTION} = $param{BODY_FUNCTION};
    } else {
        $self->{BODY_FUNCTION} = \&_comment_display;
    }

    #
    if ( $param{FOOTER} ) {
        $self->{FOOTER} = $param{FOOTER};
    } else {
        $self->{FOOTER} = <<FOOTER_END;
</body>
</html>
FOOTER_END
    }

    #
    if ( $param{PASSWORD} ) {
        $self->{PASSWORD} = $param{PASSWORD};
    } else {
        $self->{PASSWORD} = "thisisareallyreallybadpasswordtouse";
    }
    
    return $self;
}

#
#
sub handle {
    my $self = shift;

    my $mode = $self->get_mode();

    if ( $mode eq "login" ) {
        $self->login_form();
    } elsif ( $mode eq "do_login" ) {
        $self->login_verify();
    } elsif ( $mode eq "logout" ) {
            $self->logout();
    } else {
        unless ( $self->is_logged_on() ) {
            $self->login_form();
        } elsif ( $mode eq "list" ) {
            my $id = $self->get_id();
            if ( $id ) {
                $self->comment_listing( $id );
            }
            else {
                $self->comment_index();
            }
        } elsif ( $mode eq "edit" ) {
            my $id = $self->get_id();
            my $index = param("index");
            if ( $id && (defined $index)) {
                $self->comment_edit( $id,$index );
            }
            else {
                $self->comment_index();
            }
        } elsif ( $mode eq "store" ) {
            my $id = $self->get_id();
            my $index = param("index");
            if ( $id && (defined $index)) {
                $self->comment_store( $id,$index );
            }
            else {
                $self->comment_index();
            }
        } elsif ( $mode eq "delete" ) {
            my $id = $self->get_id();
            my $index = param("index");
            if ( $id && (defined $index) ) {
                $self->comment_delete( $id,$index );
            } else {
                $self->comment_index();
            }
        } elsif ( $mode eq "approve" ) {
            my $id = $self->get_id();
            my $index = param("index");
            if ( $id && (defined $index)) {
                $self->comment_approve( $id,$index );
            }
            else {
                $self->comment_index();
            }
        } elsif ( $mode eq "xml" ) {
            $self->comment_xml();
        } else {
            $self->comment_index();
        }
    }
}

#
#
sub login_form {
    my ($self,$msg) = @_;

    print <<FORM_1;
Content-type: text/html

<html>
<head><title>Comment Administration</title>
<script>
<!--
function sf(){document.f.key.focus();}
// -->
</script>
</head>
<body color="black" bgcolor="#CCCCCC" onLoad=sf()>
FORM_1
    if ( $msg ) { print "<b>$msg</b><br>\n"; }
    print <<FORM_2;
<form method="post" name="f"/>
<input type="hidden" name="$self->{MODE_PARAMETER}" value="do_login"/>
Password: <input name="key" type="password"/>
<br><input type="submit" value="Log in"/>
</form>
FORM_2
    print $self->{FOOTER};
}

sub login_verify {
    my $self = shift;
    my $key  = param('key');
    unless ($key eq $self->{PASSWORD}) {
        $self->login_form("Login Failed");
        exit;
    }
    require CGI::Cookie;
    my @alpha = ('a'..'z', 'A'..'Z', 0..9);
    my $salt = join '', map $alpha[rand @alpha], 1..2;
    my $cookie = CGI::Cookie->new(-name => 'cmKey',
        -value => crypt($key, $salt));
    my $url = url();
    print header(-cookie => $cookie),
        $self->{HEADER},
        qq(<p>Logged in</p>Proceed to <a href="$url">Comment Index</a>), $self->{FOOTER};
}

sub logout {
    my $self = shift;
    require CGI::Cookie;
    my $cookie = CGI::Cookie->new(-name => 'cmKey', -value => '',
        -expire => '-1y');
    print header(-cookie => $cookie);
    print $self->{HEADER};
    print "Logged Out";
    print $self->{FOOTER};
}

sub is_logged_on {
    my $self = shift;
    require CGI::Cookie;
    my %cookies = CGI::Cookie->fetch;
    return unless $cookies{cmKey};
    my $key = $cookies{cmKey}->value || return;
    $key eq crypt $self->{PASSWORD}, substr $key, 0, 2;
}

#
#
sub comment_xml {
    my $self = shift;

    print "Content-type: text/xml\n\n";

    my $count_data = $self->count_load();

    print "<comments>\n";
    foreach my $id (sort keys(%$count_data)) {
        my $data = $self->data_load($id);
        for my $comment(@$data) {
            my $ts = POSIX::strftime( "%d %b %Y, %I:%M %p %Z",
                                      localtime($comment->{timestamp}) );
            my $text = $comment->{excerpt};
            $text =~ s/&/&amp;/g;
            $text =~ s/</&lt;/g;
            $text =~ s/>/&gt;/g;
            print "<comment>\n";
            print " <postId>$id</postId>\n";
            print " <name>$comment->{blog_name}</name>\n";
            print " <url>$comment->{url}</url>\n";
            print " <text>$text</text>\n";
            print " <date>$ts</date>\n";
            print " <approved>$comment->{approved}</approved>\n";
            print "</comment>\n";
        }
    }
    print "</comments>\n";
}

sub comment_index {
    my $self = shift;

    my $url  = url();
    my $list = $url . "?$self->{MODE_PARAMETER}=list&$self->{ID_PARAMETER}=";

    print "Content-type: text/html\n\n";
    print $self->{HEADER};

    my $count_data = $self->count_load();
    my ($key);
    foreach $key (reverse sort keys(%$count_data)) {
        print qq(<br><a href="$list$key">$key</a> $count_data->{$key}\n);
    }

    print qq(<hr><a href="$url?$self->{MODE_PARAMETER}=logout">Logout</a>);
    print qq(&nbsp;&nbsp;&nbsp;download as <a href="$url?$self->{MODE_PARAMETER}=xml">XML</a>);
    print $self->{FOOTER};
}

sub comment_listing {
    my $self     = shift;
    my $id       = shift;
    my $data     = $self->data_load($id);
    my $url      = url();
    my $ping_url = $url . "/$id";
    my $stub_url = $url . "?$self->{ID_PARAMETER}=$id&$self->{MODE_PARAMETER}=%s&index=%s";

    print "Content-type: text/html\n\n";
    print $self->{HEADER};
    print $self->{BODY_FUNCTION}($data,$stub_url);

    print qq(<hr><a href="$url?$self->{MODE_PARAMETER}=index">Index</a> ~ <a href="$url?$self->{MODE_PARAMETER}=logout">Logout</a>);
    print $self->{FOOTER};
}

sub _comment_display {
    my ($data,$stub_url) = @_;
    
    my $tmpl = <<TEMPLATE;
<div class="%s">
<div class="comment">%s</div>
<div class="nameDate">%s &#0149; %s</div>
IP Address: %s <a href="$stub_url">EDIT</a> <a href="$stub_url">DELETE</a> <a href="$stub_url">%s</a>
</div>
TEMPLATE

    my $i = 0;
    for my $item (@$data) {
        my $ts = POSIX::strftime("%d %b %Y, %I:%M %p %Z", localtime($item->{timestamp}) );
        my $name = $item->{blog_name} || "Anonymous Coward";
        my $nameUrl = $item->{url}
                    ? qq(<a href="$item->{url}" target="_new">$name</a>)
                    : $name;
        printf $tmpl,
            ( $i % 2 ? "odd" : "even"),
            $item->{excerpt},
            $nameUrl, $ts,
            $item->{ipaddr},
            "edit", $i,
            "delete", $i,
            "approve", $i, ( $item->{approved} ? 'REVOKE?' : 'APPROVE?' );
                
        $i ++;
    }
}

#
#
sub comment_edit {
    my $self     = shift;
    my $id       = shift;
    my $index    = shift;
    my $data     = $self->data_load($id);
    my $comment  = @$data[$index];

    my $template = <<TEMPL_END;
Content-type: text/html

<html>
<head></head>
<body>
<form method="POST" target="_self">
<table>
<input type="hidden" name="%s" value="%s">
<input type="hidden" name="%s" value="store">
<input type="hidden" name="index" value="%s">
<tr><td>Name:</td>
    <td><input type="text" name="blog_name" size="50" maxlength="255" value="%s" /></td>
</tr>
<tr><td>Web site:</td>
    <td><input type="text" name="url" size="50" maxlength="255" value="%s" /></td>
</tr>
<tr><td valign="top" width="1%">Comment:</td>
    <td valign="top"><textarea name="excerpt" rows="7" cols="80" style="width:100%">
TEMPL_END

    my $conclude = <<CONCLUDE;
</textarea></td>
</tr>
<tr><td>&nbsp;</td>
    <td><input type="submit"><input type="reset"></td>
</tr>
</table>
</form>
</body>
</html>
CONCLUDE

    printf $template,
           $self->{ID_PARAMETER}, $id,
           $self->{MODE_PARAMETER},
           $index,
           $comment->{blog_name},
           $comment->{url};
    print $comment->{excerpt};
    print $conclude;
}

sub comment_store {
    my $self     = shift;
    my $id       = shift;
    my $index    = shift;
    my $data     = $self->data_load($id);

    @$data[$index]->{blog_name} = param("blog_name");
    @$data[$index]->{url}       = param("url");
    @$data[$index]->{excerpt}   = param("excerpt");

    $self->data_store($id,$data);

    print redirect(url() . "?$self->{MODE_PARAMETER}=list&$self->{ID_PARAMETER}=$id");
}

#
#
sub comment_delete {
    my ($self, $id, $index) = @_;
    my $data = $self->data_load($id);
    splice @$data, $index, 1;
    $self->data_store($id,$data);
    if ( scalar @$data > 0 ) {
        print redirect(url() . "?$self->{MODE_PARAMETER}=list&$self->{ID_PARAMETER}=$id");
    } else {
        print redirect(url());
    }
}

#
#
sub comment_approve {
    my ($self, $id, $index) = @_;
    my $data = $self->data_load($id);

    @$data[$index]->{approved}   = ( @$data[$index]->{approved} ) ? 0 : 1;
    $self->data_store($id,$data);

    print redirect(url() . "?$self->{MODE_PARAMETER}=list&$self->{ID_PARAMETER}=$id");
}

1;
