# ---------------------------------------------------------------------------
# Copyright 2002 Benjamin Trott
# Copyright 2003 Dwight Shih
#
# Functions for handling and displaying trackback pings
# Derived from the Standalone Trackback package released by Benjamin Trott.
#
# This code is released under the Artistic License.
#
# $Id: admin.pm,v 1.6 2005/03/18 23:57:22 dwight Exp $
# ---------------------------------------------------------------------------

package trackback::admin;

use strict;
use vars qw($VERSION @ISA);

require POSIX;

use Exporter;
use CGI qw( :cgi :standard );

use trackback::core;

@ISA = qw(Exporter trackback::core);
$VERSION = '1.00';

# ---------------------------------------------------------------------------

sub new {
    my $class = shift;
    my %param = @_;
    my $self  = trackback::core->new(@_);
    bless($self,$class);

    #
    if ( $param{HEADER} ) {
        $self->{HEADER} = $param{HEADER};
    } else {
        $self->{HEADER} = <<HEADER_END;
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">

<html>
<head>
<title>Trackback Administration</title>
<meta name="robots" content="noindex,nofollow">
<style type="text/css">
body {
    color:black;
    background-color:#CCCCCC;
}
.even {
    color:black;
    background-color:white;
    padding-top: 5px;
    padding-bottom: 5px;
}
.odd {
    color:black;
    background-color:#CCCCCC;
    padding-top: 5px;
    padding-bottom: 5px;
}
</style>
</head>
<body>
HEADER_END
    }

    #
    if ( $param{BODY_FUNCTION} ) {
        $self->{BODY_FUNCTION} = $param{BODY_FUNCTION};
    } else {
        $self->{BODY_FUNCTION} = \&_trackback_display;
    }

    #
    if ( $param{FOOTER} ) {
        $self->{FOOTER} = $param{FOOTER};
    } else {
        $self->{FOOTER} = <<FOOTER_END;
</body>
</html>
FOOTER_END
    }

    #
    if ( $param{PASSWORD} ) {
        $self->{PASSWORD} = $param{PASSWORD};
    } else {
        $self->{PASSWORD} = "thisisareallyreallybadpasswordtouse";
    }
    
    return $self;
}

#
#
sub handle {
    my $self = shift;

    my $mode = $self->get_mode();

    if ( $mode eq "login" ) {
        $self->login_form();
    } elsif ( $mode eq "do_login" ) {
        $self->login_verify();
    } elsif ( $mode eq "logout" ) {
            $self->logout();
    } else {
        unless ( $self->is_logged_on() ) {
            $self->login_form();
        } elsif ( $mode eq "list" ) {
            my $id = $self->get_id();
            if ( $id ) {
                $self->trackback_listing( $id );
            }
            else {
                $self->trackback_index();
            }
        } elsif ( $mode eq "delete" ) {
            my $id = $self->get_id();
            my $index = param("index");
            if ( $id && (defined $index) ) {
                $self->trackback_delete( $id,$index );
            } else {
                $self->trackback_index();
            }
        } elsif ( $mode eq "approve" ) {
            my $id = $self->get_id();
            my $index = param("index");
            if ( $id && (defined $index)) {
                $self->trackback_approve( $id,$index );
            }
            else {
                $self->trackback_index();
            }
        } elsif ( $mode eq "xml" ) {
            $self->trackback_xml();
        } else {
            $self->trackback_index();
        }
    }
}

#
#
sub login_form {
    my ($self,$msg) = @_;

    print <<FORM_1;
Content-type: text/html

<html>
<head><title>Trackback Administration</title>
<script>
<!--
function sf(){document.f.key.focus();}
// -->
</script>
</head>
<body color="black" bgcolor="#CCCCCC" onLoad=sf()>
FORM_1
    if ( $msg ) { print "<b>$msg</b><br>\n"; }
    print <<FORM_2;
<form method="post" name="f"/>
<input type="hidden" name="$self->{MODE_PARAMETER}" value="do_login"/>
Password: <input name="key" type="password"/>
<br><input type="submit" value="Log in"/>
</form>
FORM_2
    print $self->{FOOTER};
}

sub login_verify {
    my $self = shift;
    my $key  = param('key');
    unless ($key eq $self->{PASSWORD}) {
        $self->login_form("Login Failed");
        exit;
    }
    require CGI::Cookie;
    my @alpha = ('a'..'z', 'A'..'Z', 0..9);
    my $salt = join '', map $alpha[rand @alpha], 1..2;
    my $cookie = CGI::Cookie->new(-name => 'key',
        -value => crypt($key, $salt));
    my $url = url();
    print header(-cookie => $cookie),
        $self->{HEADER},
        qq(<p>Logged in</p>Proceed to <a href="$url">Trackback Index</a>), $self->{FOOTER};
}

sub logout {
    my $self = shift;
    require CGI::Cookie;
    my $cookie = CGI::Cookie->new(-name => 'key', -value => '',
        -expire => '-1y');
    print header(-cookie => $cookie);
    print $self->{HEADER};
    print "Logged Out";
    print $self->{FOOTER};
}

sub is_logged_on {
    my $self = shift;
    require CGI::Cookie;
    my %cookies = CGI::Cookie->fetch;
    return unless $cookies{key};
    my $key = $cookies{key}->value || return;
    $key eq crypt $self->{PASSWORD}, substr $key, 0, 2;
}

#
#
sub trackback_xml {
    my $self = shift;

    print "Content-type: text/xml\n\n";

    my $count_data = $self->count_load();

    print "<pings>\n";
    foreach my $id (sort keys(%$count_data)) {
        my $data = $self->data_load($id);
        for my $ping(@$data) {
            my $ts = POSIX::strftime( "%d %b %Y, %I:%M %p %Z",
                                      localtime($ping->{timestamp}) );
            my $text = $ping->{excerpt};
            $text =~ s/&/&amp;/g;
            $text =~ s/</&lt;/g;
            $text =~ s/>/&gt;/g;
            print "<ping>\n";
            print " <postId>$id</postId>\n";
            print " <name>$ping->{blog_name}</name>\n";
            print " <url>$ping->{url}</url>\n";
            print " <text>$text</text>\n";
            print " <date>$ts</date>\n";
            print " <approved>$ping->{approved}</approved>\n";
            print "</ping>\n";
        }
    }
    print "</pings>\n";
u}

sub trackback_index {
    my $self = shift;

    my $url  = url();
    my $list = $url . "?$self->{MODE_PARAMETER}=list&$self->{ID_PARAMETER}=";

    print "Content-type: text/html\n\n";
    print $self->{HEADER};

    my $count_data = $self->count_load();
    my ($key);
    foreach $key (reverse sort keys(%$count_data)) {
        print qq(<br><a href="$list$key">$key</a> [$count_data->{$key}]\n);
    }

    print qq(<hr><a href="$url?$self->{MODE_PARAMETER}=logout">Logout</a>);
    print qq(&nbsp;&nbsp;&nbsp;download as <a href="$url?$self->{MODE_PARAMETER}=xml">XML</a>);
    print $self->{FOOTER};
}

sub trackback_listing {
    my $self     = shift;
    my $id       = shift;
    my $data     = $self->data_load($id);
    my $url      = url();
    my $ping_url = $url . "/$id";
    my $stub_url = $url . "?$self->{MODE_PARAMETER}=%s&$self->{ID_PARAMETER}=$id&index=";

    print "Content-type: text/html\n\n";
    print $self->{HEADER};
    print $self->{BODY_FUNCTION}($data,$stub_url);

    print qq(<hr><a href="$url?$self->{MODE_PARAMETER}=index">Index</a> ~ <a href="$url?$self->{MODE_PARAMETER}=logout">Logout</a>);
    print $self->{FOOTER};
}

sub _trackback_display {
    my ($data,$stub_url) = @_;
    
    my $tmpl = <<TEMPLATE;
<div class="%s">
<div class="trackbackTitle"><b><a href="%s">%s</a></b></div>
<div class="trackbackExcerpt"><b>Excerpt:</b> %s</div>
<div class="trackbackDate"><b>Weblog:</b> %s &#0149; %s</div>
IP Address: %s <a href="$stub_url%s">DELETE</a> <a href="$stub_url%s">%s</a> 
</div>
TEMPLATE

    my $i = 0;
    for my $item (@$data) {
        my $ts = POSIX::strftime("%d %b %Y, %I:%M %p %Z", localtime($item->{timestamp}) );
        printf $tmpl,
            ( $i % 2 ? "odd" : "even"),
            $item->{url}, $item->{title},
            $item->{excerpt} || "[no excerpt]",
            $item->{blog_name} || "[no blog name]",
            $ts,
            $item->{ipaddr},
            "delete", $i,
            "approve", $i, ( $item->{approved} ? "REVOKE?" : "APPROVE?");
        $i ++;
    }
}

#
#
sub trackback_delete {
    my ($self, $id, $index) = @_;
    my $data = $self->data_load($id);
    splice @$data, $index, 1;
    $self->data_store($id,$data);
    if ( scalar @$data > 0 ) {
        print redirect(url() . "?$self->{MODE_PARAMETER}=list&$self->{ID_PARAMETER}=$id");
    } else {
        print redirect(url());
    }
}

#
#
sub trackback_approve {
    my ($self, $id, $index) = @_;
    my $data = $self->data_load($id);

    @$data[$index]->{approved}   = ( @$data[$index]->{approved} ) ? 0 : 1;
    $self->data_store($id,$data);

    print redirect(url() . "?$self->{MODE_PARAMETER}=list&$self->{ID_PARAMETER}=$id");
}

1;
